##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


import hashlib
import os
import posixpath
import re
import urllib
import urlparse


_md5_re = re.compile(r'md5=([a-f0-9]+)')


def _get_md5_from_url(url):
  match = _md5_re.search(url)
  if match:
    return match.group(1)
  return None


def get_directory_key(url):
    """It must retorn the directory key based on the URL.

    Basically check if the url belongs to pypi:
      - if yes, the directory key will be pypi-buildout-urlmd5
      - if not, the directory key will be slapos-buildout-urlmd5
    """
    urlmd5 = hashlib.md5(url).hexdigest()
    if 'pypi' in url:
      return 'pypi-buildout-%s' % urlmd5
    return 'slapos-buildout-%s' % urlmd5


def download_network_cached(sha_dir, sha_cache, path, url, logger, md5sum=None):
    """Download from a network cache provider

    If something fail (providor be offline, or hash_string fail), we ignore
    network cached files.

    return True if download succeeded.
    """
    try:
      from slapos.libnetworkcache import NetworkcacheClient, UploadError, \
                                   DirectoryNotFound
    except:
      return False


    if sha_cache in (None, '',):
        # Not able to use network cache
        return False
    if md5sum is None:
        md5sum = _get_md5_from_url(url)


    directory_key = get_directory_key(url)
    file_name = os.path.basename(url)

    logger.info('Downloading %s from network cache.' % file_name)
    try:
        nc = NetworkcacheClient(shacache=sha_cache, shadir=sha_dir)
        file_content = nc.select(directory_key)

        f = open(path, 'w+b')
        try:
          f.seek(0)
          f.write(file_content)
        finally:
          f.close()

        if not check_md5sum(path, md5sum):
            logger.info('MD5 checksum mismatch downloading %s' % file_name)
            return False

    except (IOError, DirectoryNotFound), e:
        logger.info('Fail to download from network cache %s: %s' % \
                                                       (file_name, str(e)))
        return False
    return True


def upload_network_cached(sha_cache, sha_dir, external_url, path, logger):
    """Upload file to a shacache server"""
    try:
      from slapos.libnetworkcache import NetworkcacheClient, UploadError, \
                                   DirectoryNotFound
    except:
      return False

    if sha_cache in [None, ''] or sha_dir in [None, '']:
        logger.debug(
             'Upload cache ignored, shacache or shadir was not provided!')
        return False

    file_name = external_url.split('/')[-1].split('#')[0].split('?')[0]
    directory_key = get_directory_key(external_url)
    kw = dict(file=file_name,
              urlmd5=hashlib.md5(external_url).hexdigest())

    f = open(path, 'r')
    try:
        nc = NetworkcacheClient(shacache=sha_cache,
                                shadir=sha_dir)
        data = nc.upload(f, directory_key, **kw)
    except (IOError, UploadError), e:
        logger.info('Fail to upload file. %s' % \
                                                  (str(e)))
        return False

    finally:
      f.close()

    return True


def get_filename_from_url(url):
    """Inspired how pip get filename from url.
    """
    url = url.split('#', 1)[0]
    url = url.split('?', 1)[0]
    url = url.rstrip('/')
    name = posixpath.basename(url)
    assert name, (
           'URL %r produced no filename' % url)
    return name


from download import check_md5sum
